@@ -29,6 +29,7 @@ from napalm.pyIOSXR import IOSXR
 from napalm.pyIOSXR.exceptions import ConnectError
 from napalm.pyIOSXR.exceptions import TimeoutError
 from napalm.pyIOSXR.exceptions import InvalidInputError
+from napalm.pyIOSXR.exceptions import XMLCLIError
 
 # import NAPALM base
 import napalm.base.helpers
@@ -158,23 +159,67 @@ class IOSXRDriver(NetworkDriver):
             "vendor": "Cisco",
             "os_version": "",
             "hostname": "",
-            "uptime": -1,
+            "uptime": -1.0,
             "serial_number": "",
             "fqdn": "",
             "model": "",
             "interface_list": [],
         }
 
-        facts_rpc_request = (
-            "<Get><Operational><SystemTime/><PlatformInventory><RackTable>"
-            "<Rack><Naming><Name>0</Name></Naming>"
-            "<Attributes><BasicInfo/></Attributes>"
-            "</Rack></RackTable></PlatformInventory></Operational></Get>"
-        )
+        facts_rpc_request = """
+<Get>
+  <Operational>
+    <SystemTime/>
+    <PlatformInventory>
+      <RackTable>
+        <Rack>
+          <Naming>
+            <Name>0</Name>
+          </Naming>
+          <Attributes>
+            <BasicInfo/>
+          </Attributes>
+        </Rack>
+      </RackTable>
+    </PlatformInventory>
+  </Operational>
+</Get>
+        """
+
+        # IOS-XR 7.3.3 and possibly other 7.X versions have this located in
+        # different location in the XML tree
+        facts_rpc_request_alt = """
+<Get>
+  <Operational>
+    <SystemTime/>
+    <Inventory>
+      <Entities>
+        <Entity>
+          <Naming>
+            <Name>Rack 0</Name>
+          </Naming>
+          <Attributes>
+            <InvBasicBag></InvBasicBag>
+          </Attributes>
+        </Entity>
+      </Entities>
+    </Inventory>
+  </Operational>
+</Get>
+"""
 
-        facts_rpc_reply = ETREE.fromstring(self.device.make_rpc_call(facts_rpc_request))
         system_time_xpath = ".//SystemTime/Uptime"
-        platform_attr_xpath = ".//RackTable/Rack/Attributes/BasicInfo"
+        try:
+            facts_rpc_reply = ETREE.fromstring(
+                self.device.make_rpc_call(facts_rpc_request)
+            )
+            platform_attr_xpath = ".//RackTable/Rack/Attributes/BasicInfo"
+        except XMLCLIError:
+            facts_rpc_reply = ETREE.fromstring(
+                self.device.make_rpc_call(facts_rpc_request_alt)
+            )
+            platform_attr_xpath = ".//Entities/Entity/Attributes/InvBasicBag"
+
         system_time_tree = facts_rpc_reply.xpath(system_time_xpath)[0]
         try:
             platform_attr_tree = facts_rpc_reply.xpath(platform_attr_xpath)[0]
@@ -185,7 +230,7 @@ class IOSXRDriver(NetworkDriver):
             str, napalm.base.helpers.find_txt(system_time_tree, "Hostname")
         )
         uptime = napalm.base.helpers.convert(
-            int, napalm.base.helpers.find_txt(system_time_tree, "Uptime"), -1
+            float, napalm.base.helpers.find_txt(system_time_tree, "Uptime"), -1.0
         )
         serial = napalm.base.helpers.convert(
             str, napalm.base.helpers.find_txt(platform_attr_tree, "SerialNumber")
@@ -221,7 +266,7 @@ class IOSXRDriver(NetworkDriver):
             "is_up": False,
             "mac_address": "",
             "description": "",
-            "speed": -1,
+            "speed": -1.0,
             "last_flapped": -1.0,
         }
 
@@ -252,12 +297,13 @@ class IOSXRDriver(NetworkDriver):
                 napalm.base.helpers.mac, raw_mac, raw_mac
             )
             speed = napalm.base.helpers.convert(
-                int,
+                float,
                 napalm.base.helpers.convert(
-                    int, napalm.base.helpers.find_txt(interface_tree, "Bandwidth"), 0
+                    float, napalm.base.helpers.find_txt(interface_tree, "Bandwidth"), 0
                 )
                 * 1e-3,
             )
+
             mtu = int(napalm.base.helpers.find_txt(interface_tree, "MTU"))
             description = napalm.base.helpers.find_txt(interface_tree, "Description")
             interfaces[interface_name] = copy.deepcopy(INTERFACE_DEFAULTS)
@@ -884,7 +930,9 @@ class IOSXRDriver(NetworkDriver):
 
         return lldp_neighbors
 
-    def cli(self, commands):
+    def cli(self, commands, encoding="text"):
+        if encoding not in ("text",):
+            raise NotImplementedError("%s is not a supported encoding" % encoding)
 
         cli_output = {}
 
