@@ -6,10 +6,13 @@ Reimplemented by ktbyers to support XML-RPC in addition to JSON-RPC
 from __future__ import print_function, unicode_literals
 
 from builtins import super
+from typing import Optional, List, Dict, Any
+
 import requests
+from requests import Response
 from requests.auth import HTTPBasicAuth
 from requests.exceptions import ConnectionError
-from requests.packages.urllib3.exceptions import InsecureRequestWarning
+from requests.packages.urllib3.exceptions import InsecureRequestWarning  # type: ignore
 import json
 
 from lxml import etree
@@ -31,13 +34,13 @@ class RPCBase(object):
 
     def __init__(
         self,
-        host,
-        username,
-        password,
-        transport="https",
-        port=None,
-        timeout=30,
-        verify=True,
+        host: str,
+        username: str,
+        password: str,
+        transport: str = "https",
+        port: Optional[int] = None,
+        timeout: int = 30,
+        verify: bool = True,
     ):
         if transport not in ["http", "https"]:
             raise NXAPIError("'{}' is an invalid transport.".format(transport))
@@ -53,16 +56,41 @@ class RPCBase(object):
         self.password = password
         self.timeout = timeout
         self.verify = verify
+        self.cmd_method: str
+        self.cmd_method_conf: str
+        self.cmd_method_raw: str
+        self.headers: Dict
 
-    def _process_api_response(self, response, commands, raw_text=False):
+    def _process_api_response(
+        self, response: Response, commands: List[str], raw_text: bool = False
+    ) -> List[Any]:
         raise NotImplementedError("Method must be implemented in child class")
 
-    def _send_request(self, commands, method):
+    def _nxapi_command_conf(
+        self, commands: List[str], method: Optional[str] = None
+    ) -> List[Any]:
+        raise NotImplementedError("Method must be implemented in child class")
+
+    def _build_payload(
+        self,
+        commands: List[str],
+        method: str,
+        rpc_version: str = "2.0",
+        api_version: str = "1.0",
+    ) -> str:
+        raise NotImplementedError("Method must be implemented in child class")
+
+    def _nxapi_command(
+        self, commands: List[str], method: Optional[str] = None
+    ) -> List[Any]:
+        raise NotImplementedError("Method must be implemented in child class")
+
+    def _send_request(self, commands: List[str], method: str) -> Response:
         payload = self._build_payload(commands, method)
 
         try:
             if not self.verify:
-                requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
+                requests.packages.urllib3.disable_warnings(InsecureRequestWarning)  # type: ignore
 
             response = requests.post(
                 self.url,
@@ -86,7 +114,7 @@ class RPCBase(object):
 
 
 class RPCClient(RPCBase):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any):
         super().__init__(*args, **kwargs)
         self.headers = {"content-type": "application/json-rpc"}
         self.api = "jsonrpc"
@@ -94,7 +122,9 @@ class RPCClient(RPCBase):
         self.cmd_method_conf = "cli"
         self.cmd_method_raw = "cli_ascii"
 
-    def _nxapi_command(self, commands, method=None):
+    def _nxapi_command(
+        self, commands: List[str], method: Optional[str] = None
+    ) -> List[Any]:
         """Send a command down the NX-API channel."""
         if method is None:
             method = self.cmd_method
@@ -107,12 +137,20 @@ class RPCClient(RPCBase):
         api_response = self._process_api_response(response, commands, raw_text=raw_text)
         return api_response
 
-    def _nxapi_command_conf(self, commands, method=None):
+    def _nxapi_command_conf(
+        self, commands: List[str], method: Optional[str] = None
+    ) -> List[Any]:
         if method is None:
             method = self.cmd_method_conf
         return self._nxapi_command(commands=commands, method=method)
 
-    def _build_payload(self, commands, method, rpc_version="2.0", api_version=1.0):
+    def _build_payload(
+        self,
+        commands: List[str],
+        method: str,
+        rpc_version: str = "2.0",
+        api_version: str = "1.0",
+    ) -> str:
         """Construct the JSON-RPC payload for NX-API."""
         payload_list = []
         id_num = 1
@@ -120,7 +158,7 @@ class RPCClient(RPCBase):
             payload = {
                 "jsonrpc": rpc_version,
                 "method": method,
-                "params": {"cmd": command, "version": api_version},
+                "params": {"cmd": command, "version": float(api_version)},
                 "id": id_num,
             }
             payload_list.append(payload)
@@ -128,7 +166,9 @@ class RPCClient(RPCBase):
 
         return json.dumps(payload_list)
 
-    def _process_api_response(self, response, commands, raw_text=False):
+    def _process_api_response(
+        self, response: Response, commands: List[str], raw_text: bool = False
+    ) -> List[Any]:
         """
         Normalize the API response including handling errors; adding the sent command into
         the returned data strucutre; make response structure consistent for raw_text and
@@ -166,10 +206,11 @@ class RPCClient(RPCBase):
 
         return new_response
 
-    def _error_check(self, command_response):
+    def _error_check(self, command_response: Dict) -> None:
         error = command_response.get("error")
         if error:
             command = command_response.get("command")
+            assert isinstance(command, str)
             if "data" in error:
                 raise NXAPICommandError(command, error["data"]["msg"])
             else:
@@ -177,7 +218,7 @@ class RPCClient(RPCBase):
 
 
 class XMLClient(RPCBase):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
         self.headers = {"content-type": "application/xml"}
         self.api = "xml"
@@ -185,7 +226,9 @@ class XMLClient(RPCBase):
         self.cmd_method_conf = "cli_conf"
         self.cmd_method_raw = "cli_show_ascii"
 
-    def _nxapi_command(self, commands, method=None):
+    def _nxapi_command(
+        self, commands: List[str], method: Optional[str] = None
+    ) -> List[Any]:
         """Send a command down the NX-API channel."""
         if method is None:
             method = self.cmd_method
@@ -199,12 +242,20 @@ class XMLClient(RPCBase):
             self._error_check(command_response)
         return api_response
 
-    def _nxapi_command_conf(self, commands, method=None):
+    def _nxapi_command_conf(
+        self, commands: List[str], method: Optional[str] = None
+    ) -> List[Any]:
         if method is None:
             method = self.cmd_method_conf
         return self._nxapi_command(commands=commands, method=method)
 
-    def _build_payload(self, commands, method, xml_version="1.0", version="1.0"):
+    def _build_payload(
+        self,
+        commands: List[str],
+        method: str,
+        xml_version: str = "1.0",
+        version: str = "1.0",
+    ) -> str:
         xml_commands = ""
         for command in commands:
             if not xml_commands:
@@ -230,7 +281,9 @@ class XMLClient(RPCBase):
         )
         return payload
 
-    def _process_api_response(self, response, commands, raw_text=False):
+    def _process_api_response(
+        self, response: Response, commands: List[str], raw_text: bool = False
+    ) -> List[Any]:
         if response.status_code not in [200]:
             msg = """Invalid status code returned on NX-API POST
 commands: {}
@@ -248,7 +301,7 @@ status_code: {}""".format(
 
         return response_list
 
-    def _error_check(self, command_response):
+    def _error_check(self, command_response: etree) -> None:
         """commmand_response will be an XML Etree object."""
         error_list = command_response.find("./clierror")
         command_obj = command_response.find("./input")
