@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright 2015 Netflix. All rights reserved.
 # Copyright 2016 BigWaveIT. All rights reserved.
+# Copyright 2022 Kirk Byers. All rights reserved.
 #
 # The contents of this file are licensed under the Apache License, Version 2.0
 # (the "License"); you may not use this file except in compliance with the
@@ -33,9 +34,11 @@ from xml.sax.saxutils import escape as escape_xml
 
 # third party lib
 from lxml import etree as ET
-from netmiko import ConnectHandler
-from netmiko.ssh_exception import NetMikoTimeoutException
-from netmiko.ssh_exception import NetMikoAuthenticationException
+from netmiko import (
+    ConnectHandler,
+    NetMikoAuthenticationException,
+    NetMikoTimeoutException,
+)
 
 # local modules
 from napalm.pyIOSXR.exceptions import LockError
@@ -47,6 +50,7 @@ from napalm.pyIOSXR.exceptions import TimeoutError
 from napalm.pyIOSXR.exceptions import IteratorIDError
 from napalm.pyIOSXR.exceptions import InvalidInputError
 from napalm.pyIOSXR.exceptions import InvalidXMLResponse
+from napalm.iosxr.utilities import strip_config_header
 
 logger = logging.getLogger(__name__)
 
@@ -59,8 +63,7 @@ class IOSXR(object):
 
     _XML_SHELL = "xml"
     _XML_MODE_PROMPT = r"XML>"
-    _READ_DELAY = 0.1  # at least 0.1, corresponding to 600 max loops (60s timeout)
-    _XML_MODE_DELAY = 1  # should be able to read within one second
+    _XML_MODE_READ_TIMEOUT = 10  # should be able to complete read within 10 seconds
 
     _ITERATOR_ID_ERROR_MSG = (
         "Non-supported IteratorID in response object. "
@@ -233,11 +236,9 @@ class IOSXR(object):
             self._xml_agent_locker.release()
 
     def _send_command_timing(self, command):
-
         return self.device.send_command_timing(
             command,
-            delay_factor=self._READ_DELAY,
-            max_loops=self._XML_MODE_DELAY / self._READ_DELAY,
+            read_timeout=self._XML_MODE_READ_TIMEOUT,
             strip_prompt=False,
             strip_command=False,
         )
@@ -275,7 +276,8 @@ class IOSXR(object):
     def _send_command(
         self,
         command,
-        delay_factor=None,
+        delay_factor=None,  # noqa
+        read_timeout=None,
         start=None,
         expect_string=None,
         read_output=None,
@@ -288,29 +290,25 @@ class IOSXR(object):
         if read_output is None:
             read_output = ""
 
-        if not delay_factor:
-            delay_factor = self._READ_DELAY
+        if not read_timeout:
+            read_timeout = self.timeout
 
         if not start:
             start = time.time()
 
         output = read_output
-
         last_read = ""
-
         if not read_output and not receive:
             # because the XML agent is able to process only one single request over the same SSH
             # session at a time first come first served
             self._lock_xml_agent(start)
             try:
-                max_loops = self.timeout / delay_factor
-                last_read = self.device.send_command_expect(
+                last_read = self.device.send_command(
                     command,
                     expect_string=expect_string,
                     strip_prompt=False,
                     strip_command=False,
-                    delay_factor=delay_factor,
-                    max_loops=max_loops,
+                    read_timeout=read_timeout,
                 )
                 output += last_read
             except IOError:
@@ -345,10 +343,10 @@ class IOSXR(object):
                         return self._send_command(
                             command,
                             expect_string=expect_string,
-                            delay_factor=delay_factor,
+                            read_timeout=read_timeout,
                         )
         else:
-            output += self._netmiko_recv()  # try to read some more
+            output += self.device._read_channel_timing()  # try to read some more
 
         if "0xa3679e00" in output or "0xa367da00" in output:
             # when multiple parallel request are made, the device throws one of the the errors:
@@ -405,15 +403,6 @@ class IOSXR(object):
         self._unlock_xml_agent()
         return str(output.replace("XML>", "").strip())
 
-    def _netmiko_recv(self):
-
-        output = ""
-
-        for tmp_output in self.device.receive_data_generator():
-            output += tmp_output
-
-        return output
-
     # previous module function __execute_rpc__
     def _execute_rpc(self, command_xml, delay_factor=0.1):
 
@@ -579,10 +568,12 @@ class IOSXR(object):
             with open(filename) as f:
                 configuration = f.read()
 
-        rpc_command = "<CLI><Configuration>{configuration}</Configuration></CLI>".format(
-            configuration=escape_xml(
-                configuration
-            )  # need to escape, otherwise will try to load invalid XML
+        rpc_command = (
+            "<CLI><Configuration>{configuration}</Configuration></CLI>".format(
+                configuration=escape_xml(
+                    configuration
+                )  # need to escape, otherwise will try to load invalid XML
+            )
         )
 
         try:
@@ -625,21 +616,14 @@ class IOSXR(object):
         show_merge = self._execute_config_show("show configuration merge")
         show_run = self._execute_config_show("show running-config")
 
-        show_merge = self.strip_config_header(show_merge)
-        show_run = self.strip_config_header(show_run)
+        show_merge = strip_config_header(show_merge)
+        show_run = strip_config_header(show_run)
 
         diff = difflib.unified_diff(
             show_run.splitlines(keepends=True), show_merge.splitlines(keepends=True)
         )
         return "".join([x.replace("\r", "") for x in diff])
 
-    @staticmethod
-    def strip_config_header(config):
-        config = re.sub(r"^Building config.*\n!! IOS.*", "", config, flags=re.M)
-        config = config.strip()
-        config = re.sub(r"^!!.*", "", config)
-        return config.strip()
-
     def compare_replace_config(self):
         """
         Compare configuration to be replaced with the one on the device.
@@ -651,7 +635,7 @@ class IOSXR(object):
         """
         diff = self._execute_config_show("show configuration changes diff")
         # Strip header lines
-        diff = self.strip_config_header(diff)
+        diff = strip_config_header(diff)
         # Strip trailer line
         diff = re.sub(r"^end$", "", diff, flags=re.M)
         return diff.strip()
@@ -719,7 +703,9 @@ class IOSXR(object):
 
         :param rb_id: Rollback a specific number of steps. Default: 1
         """
-        rpc_command = "<Unlock/><Rollback><Previous>{rb_id}</Previous></Rollback><Lock/>".format(
-            rb_id=rb_id
+        rpc_command = (
+            "<Unlock/><Rollback><Previous>{rb_id}</Previous></Rollback><Lock/>".format(
+                rb_id=rb_id
+            )
         )
         self._execute_rpc(rpc_command)
